import _extends from "@babel/runtime/helpers/extends";
import React, { forwardRef, useCallback, useMemo, useState } from 'react';
import { normaliseJqlString } from '@atlassianlabs/jql-ast';
import deburr from 'lodash/deburr';
import noop from 'lodash/noop';
import Icon from '@atlaskit/icon';
import InfoIcon from '@atlaskit/icon/glyph/editor/panel';
import { N400 } from '@atlaskit/theme/colors';
import Tooltip from '@atlaskit/tooltip';
import { useIntl } from '../../../../state';
import { checkboxGlyph, dateGlyph, dropdownGlyph, labelGlyph, numberGlyph, paragraphGlyph, peopleGlyph, shortTextGlyph, timeStampGlyph } from './glyphs';
import { messages } from './messages';
import { DeprecatedOptionContainer, FieldType, FieldTypeIcon, OptionHighlight, OptionListItem, OptionName, TooltipContent } from './styled';
const ResizedIcon = ({
  glyph
}) => /*#__PURE__*/React.createElement(Icon, {
  glyph: glyph,
  size: "small",
  label: "",
  testId: "jql-editor-field-type-icon"
});

/**
 * List of types with supported icons is derived from GIN
 * Custom SVGs are provisional while we work on making field type icons consistent across Atlassian
 */
const getFieldTypeIcon = type => {
  switch (type) {
    case 'Checkboxes':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: checkboxGlyph
      });
    case 'Date':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: dateGlyph
      });
    case 'Dropdown':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: dropdownGlyph
      });
    case 'Labels':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: labelGlyph
      });
    case 'Number':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: numberGlyph
      });
    case 'Paragraph':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: paragraphGlyph
      });
    case 'People':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: peopleGlyph
      });
    case 'Short text':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: shortTextGlyph
      });
    case 'Time stamp':
      return /*#__PURE__*/React.createElement(ResizedIcon, {
        glyph: timeStampGlyph
      });
    default:
      return null;
  }
};
const getHighlightPosition = (name, matchedText) => {
  const normalizedName = deburr(name).toLowerCase();
  const normalizedMatchedText = deburr(normaliseJqlString(matchedText)).toLowerCase();
  const index = normalizedName.indexOf(normalizedMatchedText);
  if (index === -1) {
    return null;
  }
  return [index, index + normalizedMatchedText.length];
};
const getDeprecatedTooltipMessage = deprecatedSearcherKey => {
  switch (deprecatedSearcherKey) {
    case 'com.pyxis.greenhopper.jira:gh-epic-link-searcher':
    case 'com.atlassian.jpo:jpo-custom-field-parent-searcher':
      return messages.deprecatedFieldTooltipParentReplacementMessage;
    default:
      return messages.deprecatedFieldTooltipDefaultMessage;
  }
};
const AutocompleteOption = /*#__PURE__*/forwardRef((props, ref) => {
  const {
    option: {
      id,
      name,
      fieldType,
      matchedText,
      isDeprecated = false,
      deprecatedSearcherKey
    },
    isSelected,
    onClick,
    onMouseMove
  } = props;
  const [overflows, setOverflows] = useState(false);
  const [{
    formatMessage
  }] = useIntl();
  const onNameRef = useCallback(node => {
    if (node != null) {
      setOverflows(node.offsetWidth < node.scrollWidth);
    }
  }, []);

  // TODO: verify type is i18n'd after integrating with GraphQL API

  const fieldTypeIcon = fieldType ? getFieldTypeIcon(fieldType) : null;
  const optionName = useMemo(() => {
    const highlightPosition = getHighlightPosition(name, matchedText);
    return highlightPosition ? /*#__PURE__*/React.createElement(OptionName, {
      ref: onNameRef,
      "aria-label": name
    }, name.substring(0, highlightPosition[0]), /*#__PURE__*/React.createElement(OptionHighlight, null, name.substring(...highlightPosition)), name.substring(highlightPosition[1])) : /*#__PURE__*/React.createElement(OptionName, {
      ref: onNameRef
    }, name);
  }, [onNameRef, name, matchedText]);
  const deprecatedTooltipContent = /*#__PURE__*/React.createElement(TooltipContent, null, formatMessage(getDeprecatedTooltipMessage(deprecatedSearcherKey), {
    b: text => /*#__PURE__*/React.createElement("b", null, text),
    received: name,
    parentReplacement: 'Parent'
  }));
  const ListItem = /*#__PURE__*/React.createElement(OptionListItem, _extends({
    id: id,
    "data-testid": "jql-editor-autocomplete-option",
    role: "option",
    isSelected: isSelected,
    isDeprecated: isDeprecated,
    onClick: isDeprecated ? noop : onClick,
    onMouseMove: isDeprecated ? noop : onMouseMove
  }, ref && {
    ref: ref
  }, {
    "aria-selected": isSelected
  }), isDeprecated ? /*#__PURE__*/React.createElement(DeprecatedOptionContainer, null, optionName, /*#__PURE__*/React.createElement(Tooltip, {
    content: deprecatedTooltipContent,
    position: 'right'
  }, /*#__PURE__*/React.createElement(InfoIcon, {
    testId: "jql-editor-deprecated-icon",
    label: "",
    primaryColor: `var(--ds-icon, ${N400})`
  }))) : optionName, fieldType && /*#__PURE__*/React.createElement(FieldType, null, fieldTypeIcon && /*#__PURE__*/React.createElement(FieldTypeIcon, null, fieldTypeIcon), fieldType));
  const tooltipContent = /*#__PURE__*/React.createElement(TooltipContent, null, name);
  if (overflows) {
    return /*#__PURE__*/React.createElement(Tooltip, {
      content: tooltipContent,
      position: 'right'
    }, ListItem);
  }
  return ListItem;
});
export default AutocompleteOption;