import _defineProperty from "@babel/runtime/helpers/defineProperty";
import { AbstractJastVisitor } from '@atlassianlabs/jql-ast';

/**
 * Given an AST with parse errors, this visitor returns a valid query that is equivalent for hydration purposes
 * (i.e. just fields, operators and values). Resulting query is generated on a best-effort basis and depends on
 * a successful parser error recovery.
 *
 * Example 1:
 * - Original query: "assignee in (abc-123-def"
 * - Equivalent query: "assignee in (abc-123-def)"
 *
 * Example 2:
 * - Original query: "project = EM and status in (Done, currentUser(), EMPTY) and reporter in"
 * - Equivalent query: "project = EM and status in (Done)"
 */
export class ValidQueryVisitor extends AbstractJastVisitor {
  constructor(...args) {
    super(...args);
    _defineProperty(this, "visitQuery", query => {
      if (!query.where) {
        return '';
      }
      return query.where.accept(this);
    });
    _defineProperty(this, "visitCompoundClause", compoundClause => {
      return compoundClause.clauses.map(clause => clause.accept(this)).filter(value => !!value).join(' and ');
    });
    _defineProperty(this, "visitTerminalClause", terminalClause => {
      const {
        field,
        operator,
        operand
      } = terminalClause;
      if (!operator || !operand) {
        return '';
      }
      const operandValue = operand.accept(this);
      if (!operandValue) {
        return '';
      }
      return `${field.text} ${operator.value} ${operandValue}`;
    });
    _defineProperty(this, "visitNotClause", notClause => {
      return notClause.clause.accept(this);
    });
    _defineProperty(this, "visitValueOperand", valueOperand => {
      return valueOperand.text;
    });
    _defineProperty(this, "visitListOperand", listOperand => {
      return `(${listOperand.values.map(value => value.accept(this)).filter(value => !!value).join(', ')})`;
    });
  }
  defaultResult() {
    return '';
  }
}